#
# WIFL Work Orders Product
# Copyright (C) 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472   ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#
    
import OFS.SimpleItem
from OFS.Folder import Folder
import Acquisition
import AccessControl.Role
import string
from Globals import DTMLFile, HTML, Persistent
from Acquisition import Implicit
from Products.ZSQLMethods.SQL import SQL
from OFS.DTMLDocument import DTMLDocument, addDTMLDocument
from OFS.Image import File, manage_addFile

from Products.WorkOrders.TaskObject import TaskObject
from Products.WorkOrders.WorkProject import WorkProject

def manage_addWorkOrder(self, pid, headline, priority, description,
                        work_type, userName, projectComponent,
						environment='Not Supplied', logfile='',
						howtorepeat='Unknown',
						fix='Unknown', patchfile='', confidential='False',
						version='Not Supplied',
						REQUEST={}):
	"""
	Add a WorkOrder
	"""
	
	data=self.sqlGetUserFromUserName(userName=userName)
	userId=0
	for ob in data:
		userId=int(sqlattr(ob,'user_id'))

	#
	# Get the Project Manager's details...
	#
	projManager_id=0
	projEmail=''
	compEmail=''
	data=self.sqlGetUserFromUserName(userName=self.projectManager)
	for ob in data:
		projManager_id=int(sqlattr(ob,'user_id'))
		projEmail=sqlattr(ob,'email')
    

	# Assign work order to component manager if defined.
	if projectComponent > 0:
		data=self.sqlGetOneProjectComponent(componentid=projectComponent)
		for ob in data:
			compManager_id=int(sqlattr(ob,'componentmanager'))
			moredata=self.sqlGetEngineerFromId(user_id=compManager_id)
			for ob2 in moredata:
				compEmail=sqlattr(ob2,'email')
			projManager_id=compManager_id

	self.sqlInsertNewWorkOrder(pid=pid, headline=headline,
							   priority=priority,
							   description=description,
							   work_type=work_type,
							   submitted_by=userId,
							   projectComponent=projectComponent,
							   project_manager=projManager_id,
							   environment=environment,
							   howtorepeat=howtorepeat, version=version,
							   fix=fix, confidential=confidential)

	lastIdSet=self.sqlGetLastInsertId(sequence='workorders_wid_seq')
	for results in lastIdSet:
		ID=str(sqlattr(results,'last_value'))
		obj=WorkOrder(id=ID, pid=pid,
					  headline=headline,
					  priority=priority,
					  description=description,
					  work_type=work_type,
					  submitted_by=userId,
					  projectComponent=projectComponent,
					  connection=self.connection)

		self._setObject('Workorder_'+ID,obj)
		obj=getattr(self,'Workorder_'+ID)
		if hasattr(obj,'postInitialisation'):
			obj.postInitialisation()

		patchfile.seek(0,2)
		fileLen=patchfile.tell()
		patchfile.seek(0)
		# print "Patchfile: len", fileLen
		if fileLen > 0:
			obj.addBinaryAttachment(attachmentId='Patch', data=patchfile)

		logfile.seek(0,2)
		fileLen=logfile.tell()
		logfile.seek(0)

		# print "Logfile: len", fileLen		
		if fileLen > 0:
			obj.addTextAttachment(attachmentId='LogFile', data=logfile)
	
	subject='Subject: New Work Order for Project: '+pid+'\n'
	mailBody=('  Submitted: '+userName+'\n'+
			  '   Headline: '+headline+'\n'+
			  '   Priority: '+str(priority)+'\n'+
			  '  Work Type: '+work_type+'\n'+
			  '    Project: '+pid+'\n'+
			  'Description:\n'+description+'\n'+
			  '\n')

	# Send email to Project Manager
	self.MailHost.send(
		messageText=mailBody,
		mto=projEmail,
		subject=subject,
		mfrom=projEmail)

	# Send email to component Manager
	if compEmail:
		self.MailHost.send(
			messageText=mailBody,
			mto=compEmail,
			subject=subject,
			mfrom=projEmail)
		

	if REQUEST:
		return self.index_html(self,REQUEST)
	return ''

manage_addWorkOrderForm=DTMLFile('manage_addWorkOrderForm',globals())

class WorkOrder(Folder):
	"""
	A Work Order
	"""

	meta_type='Work Order'

	manage_options = (
		{'label':'View', 'action':'index_html'},
		{'label':'Edit', 'action':'manage_editWorkOrderForm'},
		{'label':'Reassign', 'action':'manage_reassignWorkOrderForm'},
		)

	ViewTextAttachment=DTMLFile('ViewTextAttachment', globals())
	ViewLog=DTMLFile('ViewLog', globals())
	WorkOrderSummary=DTMLFile('WorkOrderSummary',globals())

	# Stupid crap...
	workOrderView=DTMLFile('workOrderView',globals())
	manage_main=DTMLFile('workOrderView',globals())
	manage_workspace=DTMLFile('workOrderView',globals())
	
	manage_editWorkOrderForm=DTMLFile('manage_editWorkOrderForm',globals())
	manage_reassignWorkOrderForm=DTMLFile('manage_reassignWorkOrderForm',globals())



	index_html=DTMLFile('index_html',globals())

	addDTMLDocument=addDTMLDocument
	manage_addFile=manage_addFile


	def __init__(self, id, pid, headline, priority, description,
				 work_type, submitted_by, projectComponent, connection,
				 ):
		self.id='Workorder_'+id
		self.wid=id
		self.pid=pid
		self.headline=headline
		self.priority=priority
		self.description=description
		self.work_type=work_type
		self.submitted_by=submitted_by
		if projectComponent > 0:
			self.projectComponent=projectComponent
		self.connection=connection
		self.addSQLQueries()

##	def __setstate__(self, state):
##		Persistent.__setstate__(self,state)
##		self.addSQLQueries()
##		self.reloadWorkOrderFromDB()
		
	def addSQLQueries(self):
		self.sqlGetStateName=SQL(
			'sqlGetStateName','Get State Name',
			self.connection,
			'state:string:required',
			_sqlGetStateName)

		self.sqlGetTransitions=SQL(
			'sqlGetTransitions','Get State Transitions',
			self.connection,
			'state:string:required',
			_sqlGetTransitions)

		self.sqlGetPriorities=SQL(
			'sqlGetPriorities', 'Get Priorities',
			self.connection,
			'',
			_sqlGetPriorities)

		self.sqlGetEngineers=SQL(
			'sqlGetEngineers', 'Get Engineers',
			self.connection,
			'',
			_sqlGetEngineers)

		self.sqlUpdateWorkOrderNew=SQL(
			'sqlUpateWorkOrderNew',
			'Update a New Work Order',
			self.connection,
			'status:required '+
			'priority:int:required '+
			'estimated_start:required '+
			'estimated_end:required '+
			'assigned_to:int:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderNew)
            
		self.sqlUpdateWorkOrderOpen=SQL(
			'sqlUpateWorkOrderOpen',
			'Update an Open Work Order',
			self.connection,
			'status:required '+
			'assigned_to:int:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderOpen)
            
		self.sqlUpdateWorkOrderAnalysed=SQL(
			'sqlUpdateWorkOrderAnalysed',
			'Update an Analysed Work Order',
			self.connection,
			'status:string:required '+
			'assigned_to:int:required '+
			'analysis:string:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderAnalysed)
        
		self.sqlUpdateWorkOrderResolved=SQL(
			'sqlUpdateWorkOrderResolved',
			'Update a Resolved Work Order',
			self.connection,
			'status:string:required '+
			'assigned_to:int:required '+
			'resolution:string:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderResolved)

		self.sqlUpdateWorkOrderVerified=SQL(
			'sqlUpdateWorkOrderVerified',
			'Update an Verified Work Order',
			self.connection,
			'status:string:required '+
			'assigned_to:int:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderVerified)

		self.sqlUpdateWorkOrderClosed=SQL(
			'sqlUpdateWorkOrderClosed',
			'Update a Closed Work Order',
			self.connection,
			'status:string:required '+
			'assigned_to:int:required '+
			'wid:int:required',
			_sqlUpdateWorkOrderClosed)

		self.sqlWorkOrderAnalyse=SQL(
			'sqlWorkOrderAnalyse',
			'Analyse a Work Order',
			self.connection,
			'wid:int:required analysed_by:int:required',
			_sqlWorkOrderAnalyse)

		self.sqlWorkOrderResolve=SQL(
			'sqlWorkOrderResolve',
			'Resolve a Work Order',
			self.connection,
			'wid:int:required resolved_by:int:required',
			_sqlWorkOrderResolve)

		self.sqlWorkOrderVerify=SQL(
			'sqlWorkOrderVerify',
			'Verify a Work Order',
			self.connection,
			'wid:int:required verified_by:int:required',
			_sqlWorkOrderVerify)

		self.sqlWorkOrderClose=SQL(
			'sqlWorkOrderClose',
			'Close a Work Order',
			self.connection,
			'wid:int:required closed_by:int:required',
			_sqlWorkOrderClose)

		self.sqlWorkOrderById=SQL(
			'sqlWorkOrderById',
			'Get One Work Order',
			self.connection,
			'wid:int:required',
			_sqlWorkOrderByIdQuery)

		self.sqlGetUserFromUserName=SQL(
			'sqlGetUserFromUserName',
			'Reverse User Mapping',
			self.connection,
			'userName:string:required',
			_sqlGetUserFromUserName)

		self.sqlGetStateList=SQL(
			'sqlGetStateList',
			'Get all States',
			self.connection,
			'',
			_sqlGetStateList)

		self.sqlGetStateCount=SQL(
			'sqlGetStateCount',
			'Get State Count',
			self.connection,
			'user_id:int:required state:string:required pid:string:required',
			_sqlGetStateCount)

		self.sqlGetPriorityCount=SQL(
			'sqlGetPriorityCount',
			'Get Priority Count',
			self.connection,
			'user_id:int:required priority:int:required pid:string:required',
			_sqlGetPriorityCount)

		self.sqlWorkOrderSummary=SQL(
			'sqlWorkOrderSummary',
			'Get Work Order Summary',
			self.connection,
			'user_id:int:required pid:string:required',
			_sqlWorkOrderSummary)

		self.sqlGetPriorityFromId=SQL(
			'sqlGetPriorityFromId',
			'Get Priority Details',
			self.connection,
			'priority:int:required',
			_sqlGetPriority)

		self.sqlGetUserDetails=SQL(
			'sqlGetUserDetails',
			'Get User Details',
			self.connection,
			'user_id:int:required',
			_sqlGetUserDetails)

		self.sqlReassignWorkOrder=SQL(
			'sqlReassignWorkOrder',
			'Assign a Work Order to an Engineer',
			self.connection,
			'assigned_to:int:required wid:int:required',
			_sqlReassignWorkOrder)

		self.sqlGetEngineerFromId=SQL(
			'sqlGetEngineerFromId',
			'Get One Engineer',
			self.connection,
			'user_id:int:required',
			_sqlGetEngineerFromId)

		self.sqlAddTextAttachment=SQL(
			'sqlAddTextAttachment',
			'Add a Text Attachment',
			self.connection,
			'id:string data:string wid:string=%s'%(self.wid),
			_sqlAddTextAttachment)

		self.sqlAddBinaryAttachment=SQL(
			'sqlAddBinaryAttachment',
			'Add a Binary Attachment',
			self.connection,
			'id:string data:string metatype:string wid:string=%s'%(self.wid),
			_sqlAddBinaryAttachment)

		self.sqlGetTextAttachments=SQL(
			'sqlGetTextAttachments',
			'Find text attachments for this Work Order',
			self.connection,
			'wid:string=%s'%(self.wid),
			_sqlGetTextAttachments)

		self.sqlGetBinaryAttachments=SQL(
			'sqlGetBinaryAttachments',
			'Find binary attachments for this Work Order',
			self.connection,
			'wid:string=%s'%(self.wid),
			_sqlGetBinaryAttachments)

		self.sqlInsertWorkOrderLogEntry=SQL(
			'sqlInsertWorkOrderLog',
			'Insert a Log Entry for this Work Order',
			self.connection,
			'wid:string message:string',
			_sqlInsertWorkOrderLogEntry)

		self.sqlListWorkOrderLogEntries=SQL(
			'sqlListWorkOrderLogEntries',
			'Get all Log Entries for a Work Order',
			self.connection,
			'wid:string',
			_sqlListWorkOrderLogEntries)

	def getPriorityName(self, priority):
		data = self.sqlGetPriorityFromId(priority=priority)
		for ob in data:
			return sqlattr(ob,'name')

	def getEngineerFromId(self,id):
		data = self.sqlGetEngineerFromId(user_id=id)
		ob = data[0]
		return ob

	def addLocalTextAttachment(self, attachmentId, data):
		self.manage_addDTMLDocument(id=attachmentId,
									title='',
									file=data)
		
	def addTextAttachment(self, attachmentId, data):
		""" Add a text attachment """

		self.addLocalTextAttachment(attachmentId, data)
		
		self.sqlAddTextAttachment(id=attachmentId,
								  data=getattr(self,attachmentId).document_src)

	def addLocalBinaryAttachment(self, attachmentId, data):
		self.manage_addFile(id=attachmentId,
							title='',
							file=data)
		
	def addBinaryAttachment(self, attachmentId, data):
		""" Add a binary attachment """

		self.addLocalBinaryAttachment(attachmentId, data)

		# Shame this doesn't quite work the way I thought it should..
##		self.sqlAddBinaryAttachment(id=attachmentId,
##									data=getattr(self,attachmentId).data,
##									metatype=getattr(self,attachmentId).getContentType())
		

	def fillNamesAndLogins(self):
		if hasattr(self,'assigned_to') and self.assigned_to:
			Engineer=self.getEngineerFromId(self.assigned_to)
			self.assigned_to_login=sqlattr(Engineer,'login')
			self.assigned_to_name=sqlattr(Engineer,'name')
		if hasattr(self,'assigned_by') and self.assigned_by:
			Engineer=self.getEngineerFromId(self.assigned_by)
			self.assigned_by_login=sqlattr(Engineer,'login')
			self.assigned_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'submitted_by') and self.submitted_by:
			Engineer=self.getEngineerFromId(self.submitted_by)
			self.submitted_by_login=sqlattr(Engineer,'login')
			self.submitted_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'analysed_by') and self.analysed_by:
			Engineer=self.getEngineerFromId(self.analysed_by)
			self.analysed_by_login=sqlattr(Engineer,'login')
			self.analysed_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'resolved_by') and self.resolved_by:
			Engineer=self.getEngineerFromId(self.resolved_by)
			self.resolved_by_login=sqlattr(Engineer,'login')
			self.resolved_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'verified_by') and self.verified_by:
			Engineer=self.getEngineerFromId(self.verified_by)
			self.verified_by_login=sqlattr(Engineer,'login')
			self.verified_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'closed_by') and self.closed_by:
			Engineer=self.getEngineerFromId(self.closed_by)
			self.closed_by_login=sqlattr(Engineer,'login')
			self.closed_by_name=sqlattr(Engineer,'name')
		if hasattr(self,'rejected_by') and self.rejected_by:
			Engineer=self.getEngineerFromId(self.rejected_by)
			self.rejected_by_login=sqlattr(Engineer,'login')
			self.rejected_by_name=sqlattr(Engineer,'name')

	def fillMandatoryFields(self, data):
		self.pid=sqlattr(data,'pid')
		self.headline=sqlattr(data,'headline')
		self.priority=sqlattr(data,'priority')
		self.priority_name=sqlattr(data,'name')
		self.description=sqlattr(data,'description')
		self.work_type=sqlattr(data,'work_type')
		self.status=sqlattr(data,'status')
		self.state_name=sqlattr(data,'state_name')
		self.submitted_by=sqlattr(data,'submitted_by')
		self.submitted_on=sqlattr(data,'submitted_on')

		#
		# Not technically mandatory, but, don't belong to any particular
		# state either
		#
		self.version=sqlattr(data,'version')
		self.environment=sqlattr(data,'environment')
		self.howtorepeat=sqlattr(data,'howtorepeat')
		self.fix=sqlattr(data,'fix')
		self.confidential=sqlattr(data,'confidential')

	def fillOpenedFields(self,data):
		if sqlattr(data,'assigned_to'):
			self.assigned_to=sqlattr(data,'assigned_to')
			self.assigned_by=sqlattr(data,'assigned_by')
			self.estimated_start=sqlattr(data,'estimated_start')
			self.estimated_end=sqlattr(data,'estimated_end')
			self.actual_start=sqlattr(data,'actual_start')
			self.actual_end=sqlattr(data,'actual_end')

	def fillAnalysedFields(self,data):
		if sqlattr(data,'analysed_on'):
			self.analysed_on=sqlattr(data,'analysed_on')
			self.analysed_by=sqlattr(data,'analysed_by')
			self.analysis=sqlattr(data,'analysis')

	def fillResolvedFields(self,data):
		if sqlattr(data,'resolved_on'):
			self.resolved_on=sqlattr(data,'resolved_on')
			self.resolved_by=sqlattr(data,'resolved_by')
			self.resolution=sqlattr(data,'resolution')

	def fillVerifiedFields(self,data):
		if sqlattr(data,'verified_on'):
			self.verified_on=sqlattr(data,'verified_on')
			self.verified_by=sqlattr(data,'verified_by')
        
	def fillClosedFields(self,data):
		if sqlattr(data,'closed_on'):
			self.closed_on=sqlattr(data,'closed_on')
			self.closed_by=sqlattr(data,'closed_by')

	def fillRejectionFields(self,data):
		if sqlattr(data,'rejected_on'):
			self.rejected_on=sqlattr(data,'rejected_on')
			self.rejected_by=sqlattr(data,'rejected_by')
			self.rejection=sqlattr(data,'rejection')
            
	def loadFromDB(self,data):
		self.fillMandatoryFields(data)
		self.fillOpenedFields(data)
		self.fillAnalysedFields(data)
		self.fillResolvedFields(data)
		self.fillVerifiedFields(data)
		self.fillClosedFields(data)
		self.fillNamesAndLogins()
		if self.work_type == 'New Work':
			self.icon='misc_/WorkOrders/newWork.gif'
		elif self.work_type == 'Change Request':
			self.icon='misc_/WorkOrders/changeRequest.gif'
		else:
			self.icon='misc_/WorkOrders/troubleReport.gif'

		self._p_changed=1

	def postInitialisation(self):
		self.reloadWorkOrderFromDB()
		self.reloadAttachments()

	def reloadWorkOrderFromDB(self):
		data=self.sqlWorkOrderById(wid=int(self.wid))
		for ob in data :
			self.loadFromDB(ob)

	def hasAttachments(self):
		""" Do we have attachments? """

		return (len(self.sqlGetTextAttachments(wid=self.wid))+
				len(self.sqlGetBinaryAttachments(wid=self.wid)))
		
	def reloadAttachments(self):
		ta = self.sqlGetTextAttachments()
		for a in ta:
			self.addLocalTextAttachment(
				attachmentId=sqlattr(a,'attachmentid'),
				data=sqlattr(a,'attachmentdata')
				)

		ba = self.sqlGetBinaryAttachments()
		for b in ba:
			self.addLocalBinaryAttachment(
				attachmentId=sqlattr(b,'attachmentid'),
				data=sqlattr(b,'attachmentdata'))
				

	def title_or_id(self):
		return self.headline
    
	def title(self):
		return self.headline

	def getUserId(self, userName):
		data=self.sqlGetUserFromUserName(userName=userName)

		for ob in data:
			return int(sqlattr(ob,'user_id'))

		# We're logged in as someone who has access to stuff,
		# but isn't in the Engineers table... probably the superuser
		return 1

	def updateWorkOrderNew(self,status, priority, assigned_to,
						   estimated_start, estimated_end):
		result=self.sqlUpdateWorkOrderNew(wid=int(self.wid), priority=priority,
										  estimated_start=estimated_start,
										  assigned_to=assigned_to,
										  estimated_end=estimated_end,
										  status=status)

	def updateWorkOrderOpen(self, status, priority, assigned_to):
		result=self.sqlUpdateWorkOrderOpen(wid=int(self.wid), status=status,
										   assigned_to=assigned_to)

	def updateWorkOrderAnalysed(self, status, assigned_to, analysed_by,
								analysis):
		result=self.sqlUpdateWorkOrderAnalysed(wid=int(self.wid),
											   assigned_to=assigned_to,
											   analysed_by=analysed_by,
											   analysis=analysis,
											   status=status)

	def updateWorkOrderResolved(self, status, assigned_to,
								resolved_by, resolution):
		result=self.sqlUpdateWorkOrderResolved(wid=int(self.wid),
											   assigned_to=assigned_to,
											   resolved_by=resolved_by,
											   resolution=resolution,
											   status=status)

	def updateWorkOrderVerified(self, status, assigned_to,
								verified_by):
		result=self.sqlWorkOrderVerified(wid=int(self.wid),
										 assigned_to=assigned_to,
										 verified_by=verified_by,
										 status=status)

	def updateWorkOrderRejected(self,status, assigned_to, rejected_by,
								rejected):
		result=self.sqlWorkOrderRejected(wid=int(self.wid),
										 assigned_to=assigned_to,
										 rejected_by=rejected_by,
										 status=status)

	def workOrderAnalyse(self,analysed_by):
		result=self.sqlWorkOrderAnalyse(wid=int(self.wid),analysed_by=analysed_by)

	def workOrderResolve(self,resolved_by):
		result=self.sqlWorkOrderResolve(wid=int(self.wid),resolved_by=resolved_by)

	def workOrderVerify(self,verified_by):
		result=self.sqlWorkOrderVerify(wid=int(self.wid), verified_by=verified_by)

	def workOrderClose(self, closed_by):
		result=self.sqlWorkOrderClose(wid=int(self.wid), closed_by=closed_by)

	def manage_reassignWorkOrder(self, assigned_to, assignmentReason,
								 assigned_by, REQUEST={}):
		"""
		Reassign a Work Order to another party
		"""

		self.assigned_by=self.getUserId(assigned_by)		
		data=self.sqlGetUserDetails(user_id=assigned_to)
		emailTo=''
		for ob in data:
			emailTo=sqlattr(ob,'email')
			subject='You have been assigned a work order\n'
			mailBody=('       Headline: '+self.headline+'\n'+
					  '       Priority: '+str(self.priority)+'\n'+
					  '      Work Type: '+self.work_type+'\n'+
					  '        Project: '+self.pid+'\n'+
					  'Was Assigned To: %s(%s)\n'%(self.assigned_to_name,
												 self.assigned_to_login)+
					  '    Description:\n'+self.description+'\n'+
					  '-----------------------------\n'+
					  assignmentReason+
					  '\n'
					  )

			# XXX FIX ME
			self.MailHost.send(
				messageText=mailBody,
				mto=emailTo, mfrom='akm@theinternet.com.au',
				subject=subject)
            
		self.sqlReassignWorkOrder(wid=int(self.wid),assigned_to=assigned_to,
								  assigned_by=assigned_by)

		# Refill all the fields
		self.reloadWorkOrderFromDB()
		self.addLogMessage('Work Order reassigned to %s'%(
			self.assigned_to_login))
		
		if REQUEST:
			return self.index_html(self,REQUEST)
		return ''

	def addLogMessage(self, logMessage):
		self.sqlInsertWorkOrderLogEntry(wid=self.wid, message=logMessage)
	
	def manage_editWorkOrder(self, status, userName, assigned_to=None,
			 priority=None,
			 estimated_start=None, estimated_end=None,
			 actual_start=None, actual_end=None,
			 analysis=None,
			 resolution=None,
			 rejection=None,
			 REQUEST={}):
		"""
		Display a form based on the current state, and the future state
		"""
		userId=self.getUserId(userName)

		self.addLogMessage('%s updated state %s'%(userName, self.status))
		if status != self.status:
			self.addLogMessage('%s changed state from %s to %s'%(userName,
																 self.status,
																 status))

			
		if self.status == 'N':
			self.updateWorkOrderNew(status, priority, userId,
									estimated_start, estimated_end)
		elif self.status == 'O':
			self.updateWorkOrderOpen(status, priority, assigned_to)
			if status == 'A':
				self.workOrderAnalyse(userId)
		elif self.status == 'A':
				self.updateWorkOrderAnalysed(status,assigned_to,
											 userId, analysis)
				if status=='R':
					self.workOrderResolve(userId)
		elif self.status == 'R':
			# print resolution
			self.updateWorkOrderResolved(status, assigned_to,
										 userId, resolution)
			if status=='V':
				self.workOrderVerify(userId)
				
		elif self.status == 'V':
			self.updateWorkOrderVerified(status, assigned_to, userId)
			if status=='C':
				self.workOrderClose(userId)
		elif self.status == 'C':
			self.updateWorkOrderClosed(status, assigned_to, userId)

		if status == 'X':
			self.workOrderReject(userId)
			self.updateWorkOrderRejected(status, assigned_to, userId,
										 rejection)

		if status != self.status:
			self.reloadWorkOrderFromDB()
			self._p_changed=1			
			if REQUEST:
				return self.manage_editWorkOrderForm(self,REQUEST)
			return ''

		self.reloadWorkOrderFromDB()
		self._p_changed=1
		
		if REQUEST:
			return self.manage_editWorkOrderForm(self,REQUEST)
		return ''


	# Over-ridable methods --
	# the default is to return the sql Method results
	def getPriorities(self):
		return self.sqlGetPriorities()

	def getTransitions(self):
		return self.sqlGetTransitions(state=self.status)

	def getStateName(self, stateName):
		return self.sqlGetStateName(state=stateName)

	def getJustStateName(self,state):
		data = self.getStateName(state)
		for ob in data:
			return sqlattr(ob,'state_name')
        
	def getEngineers(self):
		return self.sqlGetEngineers()



def loadWorkOrderFromDB(self):
	# print self.pid,"Is loading from DB"
	results=self.sqlWorkOrderQuery(pid=self.pid)
	for wo in results:
		ID=str(sqlattr(wo, 'wid'))
		headline=sqlattr(wo,'headline')
		priority=sqlattr(wo,'priority')
		description=sqlattr(wo,'description')
		work_type=sqlattr(wo,'work_type')
		submitted_by=sqlattr(wo,'submitted_by')
		projectcomponent=sqlattr(wo,'projectcomponent')
		obj=WorkOrder(id=ID, pid=self.pid,
					  headline=headline,
					  priority=priority,
					  description=description,
					  work_type=work_type,
					  submitted_by=submitted_by,
					  connection=self.connection,
					  projectComponent=projectcomponent)
		self._setObject('Workorder_'+ID,obj)
		obj=getattr(self,'Workorder_'+ID)
		if hasattr(obj,'postInitialisation'):
			obj.postInitialisation()

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name

###
### SQL QUERIES
_sqlWorkOrderByIdQuery="""
SELECT WorkOrders.*, Priorities.name, States.state_name, Engineers.login
FROM WorkOrders JOIN Priorities
ON WorkOrders.priority=Priorities.priority
JOIN States
ON States.state = WorkOrders.status
JOIN Engineers ON user_id = WorkOrders.submitted_by
WHERE WorkOrders.wid=<dtml-sqlvar wid type=int>
"""

_sqlGetUserDetails="""
SELECT * from Engineers where user_id=<dtml-sqlvar user_id type=int>
"""

_sqlGetTransitions="""
select * 
FROM State_Transitions
WHERE start_state=<dtml-sqlvar state type=string>
"""

_sqlGetStateName="""
select * 
FROM States 
WHERE state=<dtml-sqlvar state type=string>
"""

_sqlGetPriorities="""
select *
from Priorities
"""

_sqlGetEngineers="""
select *
from Engineers
"""

_sqlUpdateWorkOrderNew="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    priority=<dtml-sqlvar priority type=int>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    estimated_start=<dtml-sqlvar estimated_start type=string>,
    estimated_end=<dtml-sqlvar estimated_end type=string>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlUpdateWorkOrderOpen="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    actual_start=NOW()
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlUpdateWorkOrderAnalysed="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    analysis=<dtml-sqlvar analysis type=string>    
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlUpdateWorkOrderResolved="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    resolution=<dtml-sqlvar resolution type=string>    
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlUpdateWorkOrderVerified="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlUpdateWorkOrderRejected="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    rejected_by=<dtml-sqlvar rejected_by type=int>,
    rejection=<dtml-sqlvar rejection type=string>,
    rejected_on=NOW(),
    actual_end=NOW()
WHERE wid=<dtml-sqlvar wid type=int>
"""


_sqlUpdateWorkOrderClosed="""
UPDATE WorkOrders
SET status=<dtml-sqlvar status type=string>,
    assigned_to=<dtml-sqlvar assigned_to type=int>,
    actual_end=NOW()
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlWorkOrderAnalyse="""
UPDATE WorkOrders
SET analysed_on=NOW(),
    analysed_by=<dtml-sqlvar analysed_by type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlWorkOrderResolve="""
UPDATE WorkOrders
SET resolved_on=NOW(),
    resolved_by=<dtml-sqlvar resolved_by type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlWorkOrderVerify="""
UPDATE WorkOrders
SET verified_on=NOW(),
    verified_by=<dtml-sqlvar verified_by type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlWorkOrderClose="""
UPDATE WorkOrders
SET closed_on=NOW(),
    closed_by=<dtml-sqlvar closed_by type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlGetUserFromUserName="""
SELECT * from Engineers
WHERE login=<dtml-sqlvar userName type=string>
"""

_sqlGetStateList="""
select *
from States
"""

_sqlReassignWorkOrder="""
update WorkOrders
set assigned_to=<dtml-sqlvar assigned_to type=int>,
    assigned_by=<dtml-sqlvar assigned_by type=int>
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlGetStateCount="""
select count(*) as state_count
FROM WorkOrders
WHERE status=<dtml-sqlvar state type=string> 
AND   assigned_to=<dtml-sqlvar user_id type=int>
AND   pid=<dtml-sqlvar pid type=string>
"""

_sqlGetPriorityCount="""
select count(*) as priority_count
FROM WorkOrders
WHERE priority=<dtml-sqlvar priority type=int> 
AND   assigned_to=<dtml-sqlvar user_id type=int>
AND   pid=<dtml-sqlvar pid type=string>
"""

_sqlWorkOrderSummary="""
SELECT wid,priority,status,work_type,headline,pid
FROM WorkOrders
WHERE assigned_to = <dtml-sqlvar user_id type=int>
AND pid=<dtml-sqlvar pid type=string>
ORDER BY priority
"""

_sqlGetPriority="""
select *
FROM Priorities
WHERE priority=<dtml-sqlvar priority type=int>
"""

_sqlGetEngineerFromId="""
SELECT * from Engineers
WHERE user_id=<dtml-sqlvar user_id type=int>
"""

_sqlAddTextAttachment="""
INSERT INTO text_attachments (wid, attachmentid, attachmentdata)
VALUES (<dtml-sqlvar wid type=int>,
        <dtml-sqlvar id type=string>,
		<dtml-sqlvar data type=string>)
"""
_sqlAddBinaryAttachment="""
INSERT INTO binary_attachments (wid, attachmentid, attachmentdata, attachmentmetatype)
VALUES (<dtml-sqlvar wid type=int>,
        <dtml-sqlvar id type=string>,
		<dtml-sqlvar data type=string>,
		<dtml-sqlvar metatype type=string>)
"""

_sqlGetTextAttachments="""
SELECT * from text_attachments
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlGetBinaryAttachments="""
SELECT * from binary_attachments
WHERE wid=<dtml-sqlvar wid type=int>
"""

_sqlInsertWorkOrderLogEntry="""
INSERT INTO workorder_logs (wid, logMessage)
VALUES (<dtml-sqlvar wid type=int>,
        <dtml-sqlvar message type=string>)
"""

_sqlListWorkOrderLogEntries="""
SELECT * from workorder_logs
WHERE wid=<dtml-sqlvar wid type=int>
ORDER BY logtime
"""

OurTask=TaskObject(name='Work Order',
                   description='An item of work with states',
				   taskClass=WorkOrder,
				   addForm=manage_addWorkOrderForm,
				   addMethod=manage_addWorkOrder,
				   loadMethod=loadWorkOrderFromDB,
				   meta_type='Work Order')
WorkProject.TaskTypes['WorkOrder']=OurTask
