#
# Work Orders Product
#
# (C) Copyright 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
from ImageFile import ImageFile

from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import HTMLFile, HTML
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from Engineer import Engineer, manage_addEngineer, manage_addEngineerForm
from Products.exUserFolder.exUserFolder import exUserFolder, manage_addexUserFolder
from Products.MailHost.MailHost import manage_addMailHost, MailHost

def manage_addWorkProject(self, id, title, description, projectManager,
						  connection,REQUEST=None):
	"""
	Add a WorkProject
	"""

	obj=WorkProject(id,title,description,projectManager,connection)
	self._setObject(id,obj)

	# This is a fairly robust method of doing this
	# And it doesn't require piss farting around with
	# ever changing APIs for transactions etc (although it might
	# trigger one transparently -- like I care).
	obj=getattr(self, id)
	if hasattr(obj, 'postInitialisation'):
		obj.postInitialisation()
		
	if REQUEST:
		return self.manage_main(self,REQUEST)
	return ''

manage_addWorkProjectForm=HTMLFile('addWorkProject', globals())

class WorkProject(Folder):
	"""
	A Project Tracker
	"""
	meta_type='Project Tracker'

	icon=ImageFile('projectIcon.gif',globals())
	
	manage_options = (
		{'label':'Contents', 'action':'manage_main'},
		{'label':'View', 'action':'index_html'},
		{'label':'Edit', 'action':'manage_editProjectForm'},
		{'label':'Add Components', 'action':'manage_addProjectComponentForm'},
		{'label':'New Work Order', 'action':'manage_addWorkOrderForm'},
		{'label':'Engineers', 'action':'manage_addEngineerForm'},
		)


	__ac_permissions__=(
		('View management screens',('manage_tabs', 'manage_main',)),
		('Change permissions', ('manage_access')),
		('Change Work Project',('manage_edit','manage_addEngineerForm')),
		('View',('',)),
		)

	manage_addProjectComponentForm=HTMLFile('manage_addProjectComponentForm',globals())
	showWorkOrders=HTMLFile('showWorkOrders',globals())
	index_html=HTMLFile('index_html',globals())
	manage_editProjectForm=HTMLFile('workProjectEdit',globals())
	WorkOrderSummary=HTMLFile('WorkOrderSummary',globals())
	UserSummary=HTMLFile('UserSummary',globals())
	manage_tabs=HTMLFile('common/manage_tabs',globals())
	DialogHeader=HTMLFile('common/DialogHeader',globals())
	DialogFooter=HTMLFile('common/DialogFooter',globals())
	standard_html_header=HTMLFile('common/standard_html_header',globals())
	standard_html_footer=HTMLFile('common/standard_html_footer',globals())
	manage_browse=HTMLFile('common/manage_browse', globals())
	smallSphere=ImageFile('common/smallSphere.gif', globals())
	manage_page_header=HTMLFile('common/manage_page_header', globals())
	manage_page_footer=HTMLFile('common/manage_page_footer', globals())	
	TaskTypes={}
	meta_types=[]
	all_meta_types=[]

	manage_addexUserFolder=manage_addexUserFolder
	manage_addMailHost=manage_addMailHost
	manage_addEngineer=manage_addEngineer
	manage_addEngineerForm=manage_addEngineerForm

	
	def __init__(self, id, title,description, projectManager,
				 connection):
		self.id=id
		self.pid=id
		self.title=title
		self.projectName=title
		self.description=description
		self.projectManager=projectManager
		self.connection=connection

		#
		# Add a Role for our acl_users
		#
		self._addRole('Engineer')
		self.addSQLQueries()
		
		
	def postInitialisation(self):
		# Create a User Folder for engineers
		REQUEST={}
		REQUEST['pgauth_connection']=self.connection
		REQUEST['pgprop_connection']=self.connection
		REQUEST['pgauth_table']='passwd'
		REQUEST['pgauth_usernameColumn']='username'
		REQUEST['pgauth_passwordColumn']='password'
		REQUEST['pgauth_rolesColumn']='roles'
		self.manage_addexUserFolder(authId='pgAuthSource',
									propId='pgPropSource',
									cookie_mode=1,
									REQUEST=REQUEST)
		
		self.manage_addMailHost(id='MailHost', title='Mail Host for Work Orders', smtp_host='localhost')

		meta_types=[]
		for os in self.TaskTypes.keys():
			setattr(self,'manage_add%sForm'%(os),
					self.TaskTypes[os].manage_addTaskForm)
			setattr(self,'manage_add%s'%(os),
					self.TaskTypes[os].manage_addTask)
			setattr(self,'manage_load%sFromDB'%(os),
					self.TaskTypes[os].loadMethod)
			meta_types.append(
				{'name':self.TaskTypes[os].meta_type,
				 'action':'manage_add%sForm'%(os)}
				 )
		self.meta_types=meta_types
		self.all_meta_types=self.meta_types
		self.loadFromDB()

	def loadFromDB(self):
		for os in self.TaskTypes.keys():
			loadMethod=getattr(self,'manage_load%sFromDB'%(os),
					self.TaskTypes[os].loadMethod)
			loadMethod(self=self)

	def buildAddMenu(self):
		""" Build a Menu String """
		menustring=''
		# Find our root
		root=getattr(self,self.pid)
		root_url=root.absolute_url()
		for os in self.TaskTypes.keys():
			menustring=(
				menustring +
				'<a href="%s/manage_add%sForm">Add New %s</a>&nbsp;|&nbsp;'%(
				root_url,os,os))
		return menustring
		
	def addSQLQueries(self):
		self.sqlGetLastInsertId=SQL(
			'sqlGetLastInsertId',
			'Get ID of last inserted site',
			self.connection,
			'sequence:string',
			_sqlGetLastInsertId)
		
		self.sqlWorkOrderQuery=SQL(
			'sqlWorkOrderQuery', 'Get Work Orders',
			self.connection,
			'pid:required',_sqlWorkOrderQuery)

		self.sqlAddProjectQuery=SQL(
			'sqlAddProjectQuery',
			'Insert a Project',
			self.connection,
			'project_code:required '+
			'project_name:required '+
			'project_manager:optional '+
			'description', _sqlAddProjectQuery)

		self.sqlWorkOrderByIdQuery=SQL(
			'sqlWorkOrderByIdQuery',
			'Get One Work Order',
			self.connection,
			'wid:int:required',
			_sqlWorkOrderByIdQuery)

		self.sqlGetEngineerFromId=SQL(
			'sqlGetEngineerFromId',
			'Get One Engineer',
			self.connection,
			'user_id:int:required',
			_sqlGetEngineerFromId)

		self.sqlInsertNewWorkOrder=SQL(
			'sqlInsertNewWorkOrder',
			'Insert a New Work Order',
			self.connection,
			'pid:string:required '+
			'headline:string:required '+
			'priority:int:required '+
			'description:string:required '+
			'work_type:string:required '+
			'submitted_by:int:required '+
			'project_manager:int:required '+
			'environment:string fix:string howtorepeat:string '+
			'version:string confidential:int',
			_sqlInsertNewWorkOrder)

		self.sqlGetUserFromUserName=SQL(
			'sqlGetUserFromUserName',
			'Reverse User Mapping',
			self.connection,
			'userName:string:required',
			_sqlGetUserFromUserName)

		self.sqlGetStateList=SQL(
			'sqlGetStateList',
			'Get all States',
			self.connection,
			'',
			_sqlGetStateList)

		self.sqlGetStateCount=SQL(
			'sqlGetStateCount',
			'Get State Count',
			self.connection,
			'user_id:int:required state:string:required pid:string:requried',
			_sqlGetStateCount)
		self.sqlGetPriorityCount=SQL(
			'sqlGetPriorityCount',
			'Get Priority Count',
			self.connection,
			'user_id:int:required priority:int:required pid:string:required',
			_sqlGetPriorityCount)

		self.sqlWorkOrderSummary=SQL(
			'sqlWorkOrderSummary',
			'Get Work Order Summary',
			self.connection,
			'user_id:int:required pid:string:required',
			_sqlWorkOrderSummary)

		self.sqlGetPriorityFromId=SQL(
			'sqlGetPriorityFromId',
			'Get Priority Details',
			self.connection,
			'priority:int:required',
			_sqlGetPriority)

		self.sqlGetPriorities=SQL(
			'sqlGetPriorities', 'Get Priorities',
			self.connection,
			'',
			_sqlGetPriorities)

		self.sqlGetTransitions=SQL(
			'sqlGetTransitions','Get State Transitions',
			self.connection,
			'state:string:required',
			_sqlGetTransitions)

		self.sqlGetStateName=SQL(
			'sqlGetStateName','Get State Name',
			self.connection,
			'state:string:required',
			_sqlGetStateName)

		self.sqlGetEngineers=SQL(
			'sqlGetEngineers',
			'Get Engineers',
			self.connection,
			'',
			_sqlGetEngineers)

		self.sqlInsertNewComponent=SQL(
			'sqlInsertNewComponent',
			'Insert a New Component',
			self.connection,
			'pid:string componentName:string componentManager:int',
			_sqlInsertNewComponent)

		self.sqlGetProjectComponents=SQL(
			'sqlGetProjectComponents',
			'Get all Components for this Project',
			self.connection,
			'pid:string',
			_sqlGetProjectComponents)

		self.sqlGetOneProjectComponent=SQL(
			'sqlGetOneProjectComponent',
			'Get One Project Component',
			self.connection,
			'componentid:int',
			_sqlGetOneProjectComponent)

	def __setstate__(self, state):
		Persistent.__setstate__(self,state)
		self.addSQLQueries()
		
	def getPid(self):
		return self.pid

	def getUserId(self, userName):
		data=self.sqlGetUserFromUserName(userName=userName)
		for ob in data:
			return int(sqlattr(ob,'user_id'))

		# We're logged in as someone who has access to stuff,
		# but isn't in the Engineers table... probably the superuser
		return 1

	def getPriorityName(self, priority):
		data = self.sqlGetPriorityFromId(priority=priority)
		for ob in data:
			return sqlattr(ob,'name')

	def getPriorities(self):
		return self.sqlGetPriorities()

	def getTransitions(self):
		return self.sqlGetTransitions(state=self.status)

	def getStateName(self, stateName):
		return self.sqlGetStateName(state=stateName)

	def getJustStateName(self,state):
		data = self.getStateName(state)
		for ob in data:
			return sqlattr(ob,'state_name')
        
	def getEngineers(self):
		return self.sqlGetEngineers()

	def manage_addProjectComponent(self, component, componentManager,
							REQUEST={}):
		"""
		Add a New Component
		"""
		componentManagerId=self.getUserId(componentManager)

		self.sqlInsertNewComponent(pid=self.pid,
								   componentName=component,
								   componentManager=componentManagerId)

		if REQUEST:
			return self.manage_main(self,REQUEST)
		return ''
		
	def manage_editProject(self, title, description, projectManager,
			 REQUEST=None):
		"""
		Edit the Work Project
		"""
		self.title=title
		self.description=description
		self.projectManager=projectManager
		self._parse()

		if REQUEST:
			return self.manage_editProjectForm(self, REQUEST)
		return ''

	def getEngineerFromId(self,id):
		data = self.sqlGetEngineerFromId(user_id=id)
		ob = data[0]
		return ob

	def fillNamesAndLogins(self,workOrder,ob):
		if sqlattr(ob,'assigned_to'):
			Engineer=self.getEngineerFromId(ob.assigned_to)
			workOrder.assigned_to_login=sqlattr(Engineer,'login')
			workOrder.assigned_to_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'assigned_by'):
			Engineer=self.getEngineerFromId(ob.assigned_by)
			workOrder.assigned_by_login=sqlattr(Engineer,'login')
			workOrder.assigned_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'submitted_by'):
			Engineer=self.getEngineerFromId(ob.submitted_by)
			workOrder.submitted_by_login=sqlattr(Engineer,'login')
			workOrder.submitted_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'analysed_by'):
			Engineer=self.getEngineerFromId(ob.analysed_by)
			workOrder.analysed_by_login=sqlattr(Engineer,'login')
			workOrder.analysed_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'resolved_by'):
			Engineer=self.getEngineerFromId(ob.resolved_by)
			workOrder.resolved_by_login=sqlattr(Engineer,'login')
			workOrder.resolved_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'verified_by'):
			Engineer=self.getEngineerFromId(ob.verified_by)
			workOrder.verified_by_login=sqlattr(Engineer,'login')
			workOrder.verified_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'closed_by'):
			Engineer=self.getEngineerFromId(ob.closed_by)
			workOrder.closed_by_login=sqlattr(Engineer,'login')
			workOrder.closed_by_name=sqlattr(Engineer,'name')
		if sqlattr(ob,'rejected_by'):
			Engineer=self.getEngineerFromId(ob.rejected_by)
			workOrder.rejected_by_login=sqlattr(Engineer,'login')
			workOrder.rejected_by_name=sqlattr(Engineer,'name')

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name
        

###
### SQL QUERIES
###

_sqlGetEngineerFromId="""
SELECT * from Engineers
WHERE user_id=<dtml-sqlvar user_id type=int>
"""

_sqlWorkOrderByIdQuery="""
SELECT WorkOrder.*, Priorities.name, States.state_name, Engineers.login
FROM WorkOrders JOIN Priorities
ON WorkOrders.priority=Priorities.priority
JOIN States
ON States.state = WorkOrders.status
JOIN Engineers ON user_id = WorkOrders.submitted_by
WHERE WorkOrders.wid=<dtml-sqlvar wid type=int>
AND WorkOrders.pid=<dtml-sqlvar pid type=string>
"""


_sqlWorkOrderQuery="""
SELECT WorkOrders.wid, WorkOrders.priority, WorkOrders.submitted_by,
       WorkOrders.status, WorkOrders.work_type, WorkOrders.headline,
       Priorities.name, States.state_name, WorkOrders.projectComponent
FROM WorkOrders JOIN Priorities
ON WorkOrders.priority=Priorities.priority
JOIN States
ON States.state = WorkOrders.status
<dtml-sqlgroup required where>
<dtml-sqltest pid column="pid" type=string>
</dtml-sqlgroup>
ORDER BY WorkOrders.priority
"""

_sqlAddProjectQuery="""
INSERT INTO Project
VALUES (NULL,
<dtml-sqlvar project_code type=string>,
<dtml-sqlvar project_name type=string>,
<dtml-sqlvar project_manager type=string>,
<dtml-sqlvar description type=string>
)
"""

_sqlInsertNewWorkOrder="""
INSERT INTO WorkOrders
(pid, headline, priority, description, work_type, submitted_on,
 submitted_by, status, assigned_to, environment, fix, howtorepeat,
 confidential, version)
 VALUES (<dtml-sqlvar pid type=string>,
 <dtml-sqlvar headline type=string>,
 <dtml-sqlvar priority type=int>,
 <dtml-sqlvar description type=string>,
 <dtml-sqlvar work_type type=string>,
 NOW(), <dtml-sqlvar submitted_by type=int>, 'N',
 <dtml-sqlvar project_manager type=int>,
 <dtml-sqlvar environment type=string>,
 <dtml-sqlvar fix type=string>,
 <dtml-sqlvar howtorepeat type=string>,
 <dtml-sqlvar confidential type=string>, 
 <dtml-sqlvar version type=string>
 )
"""

_sqlGetUserFromUserName="""
SELECT * from Engineers
WHERE login=<dtml-sqlvar userName type=string>
"""


_sqlGetPriorityCount="""
select count(*) as priority_count
FROM WorkOrders
WHERE priority=<dtml-sqlvar priority type=int> 
AND   assigned_to=<dtml-sqlvar user_id type=int>
AND   pid=<dtml-sqlvar pid type=string>
"""

_sqlWorkOrderSummary="""
SELECT wid,priority,status,work_type,headline
FROM WorkOrders
WHERE assigned_to = <dtml-sqlvar user_id type=int>
AND pid=<dtml-sqlvar pid type=string>
ORDER BY priority
"""

_sqlGetPriority="""
select *
FROM Priorities
WHERE priority=<dtml-sqlvar priority type=int>
"""

_sqlGetStateList="""
select *
from States
"""

_sqlGetStateCount="""
select count(*) as state_count
FROM WorkOrders
WHERE status=<dtml-sqlvar state type=string> 
AND   assigned_to=<dtml-sqlvar user_id type=int>
AND   pid=<dtml-sqlvar pid type=string>
"""

_sqlGetPriorities="""
select *
from Priorities
"""

_sqlGetTransitions="""
select * 
FROM State_Transitions
WHERE start_state=<dtml-sqlvar state type=string>
"""

_sqlGetStateName="""
select * 
FROM States 
WHERE state=<dtml-sqlvar state type=string>
"""

_sqlGetEngineers="""
select *
from Engineers
"""
_sqlGetLastInsertId="""
select last_value FROM <dtml-var sequence>
"""

_sqlInsertNewComponent="""
INSERT INTO projectcomponents (pid, componentname, componentmanager)
VALUES (<dtml-sqlvar pid type=string>,
        <dtml-sqlvar componentName type=string>,
		<dtml-sqlvar componentManager type=int>)
"""

_sqlGetProjectComponents="""
SELECT * from ProjectComponents
WHERE pid=<dtml-sqlvar pid type=string>
"""

_sqlGetOneProjectComponent="""
SELECT * from ProjectComponents
WHERE componentid=<dtml-sqlvar componentid type=int>
"""

"""
#>Submitter-Id:  current-users
#>Originator:    Andrew Milton
#>Organization:  <organization of PR author (multiple lines)>
#>Confidential:  no <FreeBSD PRs are public data>
#>Synopsis:      <synopsis of the problem (one line)>
#>Severity:      <[ non-critical | serious | critical ] (one line)>
#>Priority:      <[ low | medium | high ] (one line)>
#>Category:      <choose from the list of categories above (one line)>
#>Class:         <[ sw-bug | doc-bug | change-request | update | maintainer-upd!
#>Release:       FreeBSD 5.0-CURRENT i386
#>Environment:
#System: FreeBSD janus.theinternet.com.au 5.0-CURRENT FreeBSD 5.0-CURRENT #9: S!


#        <machine, os, target, libraries (multiple lines)>
#>Description:
#        <precise description of the problem (multiple lines)>
#>How-To-Repeat:
#        <code/input/activities to reproduce the problem (multiple lines)>
#>Fix:
#       <how to correct or work around the problem, if known (multiple lines)>
"""
